/*

    Program: asciireflector - simple ASCII based net-reflector
    Copyright (C) 1999  A. Schiffler

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    To contact the author try these:
    Andreas Schiffler, 
    ZKM, Lorezstr. 19, 76135 Karlsruhe, Germany
    schiffler@zkm.de

*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "refsocket.h"

#define PORT		1522
#define CONNECTIONS	20

/* Converts ascii text to in_addr struct */

struct in_addr *atoaddr(address)
char *address;
{
  struct hostent *host;
  static struct in_addr saddr;

  /* First try it as aaa.bbb.ccc.ddd. */
  saddr.s_addr = inet_addr(address);
  if (saddr.s_addr != -1) {
    return &saddr;
  }
    
  host = gethostbyname(address);
  if (host != NULL) {
    return (struct in_addr *) *host->h_addr_list;
  }
  return NULL;
}

int main (int argc, char *argv[])
{
 struct netthreads nt;
 unsigned short socket_port;
 struct sockaddr_in address;
 int listening_socket = -1;
 int connected_socket = -1;
 int in_loop;
 char hostname[128];
 int length;
 int value;
  
 fprintf (stderr,"asciireflector V1.2 - A. Schiffler, 10/1999\n\n");
 
 /* Read socket port from commandline */
 if (argc>1) {  
  socket_port=atoi(argv[1]);
 } else {
  socket_port=PORT;
 }

 gethostname(hostname,sizeof(hostname));
 address.sin_family = AF_INET;
 address.sin_port = htons(socket_port);
 address.sin_addr.s_addr = INADDR_ANY;

 /* Create listening socket */
 listening_socket = socket(AF_INET, SOCK_STREAM, 0);
 if (listening_socket < 0) {
  perror("socket");
  exit(-1);
 }
 /* Set options for socket */
 value=1;
 setsockopt(listening_socket,SOL_SOCKET,SO_REUSEADDR,&value,sizeof(value));
 /* Bind socket to address */
 if (bind(listening_socket,&address,sizeof(address)) < 0) {
  perror("bind");
  close(listening_socket);
  exit(-1);
 }
 /* Allow multiple connections */
 listen(listening_socket, CONNECTIONS);

 /* Initialize mutex and variables */
 if (init_network_socket(&nt)!=0) exit(-1);
 
 /* Get a connection and handle each with a seperate thread */
 in_loop=1;
 while (in_loop) {
  fprintf (stderr,"Waiting for connection on port %i ...\n",socket_port);
  length=sizeof(address);
  connected_socket = accept(listening_socket, &address, &length);
  if ((connected_socket < 0) && (errno != EINTR)) {
   perror("accept");
   in_loop=0;
  } else {
   /* Store file handle of connected socket and create thread */
   if (start_network_socket(&nt, connected_socket)!=0) exit(-1);
   fprintf (stderr,"Got connection #%i ...\n",connected_socket);
  }
 }
 close(listening_socket);
 exit(0);
}
